package com.gmh.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gmh.entity.GmhUser;
import com.gmh.entity.R;
import com.gmh.entity.RowCellNum;
import com.gmh.entity.SysBaijiaxing;
import com.gmh.entity.vo.ReadNameVo;
import com.gmh.service.SysBaijiaxingService;
import com.gmh.utils.CommonUtils;
import com.gmh.utils.POIExcelUtil;
import com.gmh.utils.RedisUtils;
import com.gmh.utils.RedisUtilsDefault;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.CellReference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.Pattern;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.*;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/excel")
public class ExcelController {

    private Integer templateSheetNum = null;

    private Workbook templateWorkbook = null;

    private List<ReadNameVo> readNameVoList = null;

    private static final String KEY_XINGMING = "姓名";

    @Value("${gmh.upload-path}")
    private String uploadPath;

    @RequestMapping("/t01")
    public R test01() {
        return R.ok("ok");
    }

    @RequestMapping("/readData")
    public R readSourceData(MultipartFile file) throws IOException {
        Workbook workbook = POIExcelUtil.readExcelFromInputStream(file.getInputStream(), file.getOriginalFilename());
        readNameVoList = new ArrayList<>();
        for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
            ReadNameVo readNameVo = new ReadNameVo();
            Sheet sheet = workbook.getSheetAt(i);
            List<Map<String, String>> excelMaps = POIExcelUtil.toListMap(sheet);
            if (excelMaps.isEmpty()) {
                continue;
            }
            Map<String, String> firstElement = excelMaps.get(0);
            String xmKey = KEY_XINGMING;
            for (String key : firstElement.keySet()) {
                if (KEY_XINGMING.equals(firstElement.get(key))) {
                    xmKey = key;
                }
            }
            if (firstElement.containsKey(xmKey)) {
                if (!KEY_XINGMING.equals(xmKey)) {
                    excelMaps.remove(firstElement);
                }
                readNameVo = getReadNameDataForNameKey(excelMaps, xmKey);
            } else {
                Map<String, Integer> map = new HashMap<>();
                for (Map<String, String> excelMap : excelMaps) {
                    for (String key : excelMap.keySet()) {
                        String value = excelMap.get(key);
                        if (baijiaxingService.likeChineseName(value)) {
                            Integer count = map.getOrDefault(key, 0);
                            map.put(key, ++count);
                        }
                    }
                }
                List<Map.Entry<String,Integer>> list = new ArrayList<>(map.entrySet());
//                list.sort(Comparator.comparingInt(Map.Entry::getValue));        //升序
                list.sort((o1, o2) -> (o2.getValue() - o1.getValue()));
                String key = list.get(0).getKey();
                readNameVo = getReadNameDataForNameKey(excelMaps, key);
            }
            readNameVo.setSheetName(sheet.getSheetName());
            readNameVoList.add(readNameVo);
        }

        return R.ok().setData(readNameVoList);
    }

    public ReadNameVo getReadNameDataForNameKey(List<Map<String, String>> excelMaps, final String key) {
        ReadNameVo readNameVo = new ReadNameVo();
        // 去重前
        List<GmhUser> oldNameList = excelMaps.stream().map(map -> new GmhUser(map.get(key))).collect(Collectors.toList());
        // 去重后
        LinkedHashSet<GmhUser> newNameList = new LinkedHashSet<>(oldNameList);
        readNameVo.setNameList(newNameList);
        // 重复检测
        int repeatCount = oldNameList.size() - newNameList.size();
        if (repeatCount > 0) {
            readNameVo.setHasRepeat(true);
            readNameVo.setRepeatCount(repeatCount);
        } else {
            readNameVo.setHasRepeat(false);
            readNameVo.setRepeatCount(0);
        }
        return readNameVo;
    }

    @RequestMapping("/template")
    public R templateUpload(MultipartFile file, String templateSheetNum) throws IOException {
        Workbook workbook = POIExcelUtil.readExcelFromInputStream(file.getInputStream(), file.getOriginalFilename());
        this.templateSheetNum = Integer.parseInt(templateSheetNum) - 1;
        Sheet sheet = workbook.getSheetAt(this.templateSheetNum);
        if (sheet == null) {
            throw new RuntimeException("sheet页不存在");
        }
        this.templateWorkbook = workbook;
        return R.ok();
    }

    @RequestMapping("/cellNumSearch")
    public R analysisCell(@Validated RowCellNum rowCellNum) {
        if (this.templateWorkbook == null) {
            return R.error("模板文件出问题了，请重新上传");
        }
        Sheet sheet = this.templateWorkbook.getSheetAt(this.templateSheetNum);
        rowCellNum = POIExcelUtil.analysisCellRowNum(rowCellNum);
        Row row = sheet.getRow(rowCellNum.getRowNum());
        if (row == null) {
            return R.ok().setData("");
        }
        Cell cell = row.getCell(rowCellNum.getCellNum());
        if (cell == null) {
            return R.ok().setData("");
        }
        String stringCellVal = POIExcelUtil.getStringCellVal(cell);
        if (StrUtil.isEmpty(stringCellVal)) {
            return R.ok().setData("");
        }
        return R.ok().setData(stringCellVal);
    }

    @RequestMapping("/executeTemplate")
    public void executeTemplate(String newCellData, String cellRowStr, HttpServletResponse response) throws IOException {
        String expr = newCellData.substring(newCellData.indexOf("@{"), newCellData.indexOf("}") + 1);
        String exprVal = newCellData.substring(newCellData.indexOf("@{") + 2, newCellData.indexOf("}"));
        String[] exprArr = exprVal.split("\\.");
        String sheetName = exprArr[0];
        String field = exprArr[1];

        Optional<ReadNameVo> first = readNameVoList
                .stream()
                .filter(item -> item.getSheetName().equalsIgnoreCase(sheetName))
                .findFirst();
        if (!first.isPresent()) {
            throw new RuntimeException("未找到sheet页，可能是表达式错误");
        }
        ReadNameVo readNameVo = first.get();
        POIExcelUtil.removeModelSheet(this.templateWorkbook, this.templateSheetNum);
        POIExcelUtil.batchCloneSheet(this.templateWorkbook, this.templateSheetNum, readNameVo.getNameList().size());
        ArrayList<GmhUser> gmhUsers = new ArrayList<>(readNameVo.getNameList());
        RowCellNum rowCellNum = POIExcelUtil.analysisCellRowNum(cellRowStr);
        for (int i = 0; i < this.templateWorkbook.getNumberOfSheets(); i++) {
            Sheet sheet = templateWorkbook.getSheetAt(i);
            if (sheet == null) {
                continue;
            }
            Row row = sheet.getRow(rowCellNum.getRowNum());
            Cell cell = row.getCell(rowCellNum.getCellNum());
            cell.setCellValue(newCellData.replace(expr, CommonUtils.nameFormat(gmhUsers.get(i).getName())));
        }

        // TODO 文件保存到本地，文件信息入库

        // 输出Excel文件
        OutputStream output = response.getOutputStream();
        response.reset();
        // 设置文件头
        response.setHeader("Content-Disposition",
                "attchement;filename=" + new String((sheetName + ".xls").getBytes("gb2312"), "ISO8859-1"));
        response.setContentType("application/msexcel");
        this.templateWorkbook.write(output);
        this.templateWorkbook.close();

//        try {
//            FileOutputStream out = new FileOutputStream("C:\\Users\\admin\\Desktop\\tempfile\\output\\out的222.xls");
//            out.flush();
//            templateWorkbook.write(out);
//            out.close();
//            System.out.println("文件输出完成");
//        } catch (IOException e) {
//            e.printStackTrace();
//        }
    }

    @Autowired
    private SysBaijiaxingService baijiaxingService;

    private boolean isChinaName(String val) {
        QueryWrapper<SysBaijiaxing> queryWrapper = new QueryWrapper<>();
        queryWrapper.apply("{0} LIKE CONCAT(xingshi,'%')", val);
        List<SysBaijiaxing> list = baijiaxingService.list(queryWrapper);
        return !list.isEmpty();
    }

    public void oleCode() {
//        Integer nameCellIndex = null;
//        // Sheet:nameList
//        List<ReadNameVo> readNameList = new ArrayList<>();
//        for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
//            List<GmhUser> nameList = new ArrayList<>();
//            Sheet sheet = workbook.getSheetAt(i);
//            for (int r = 0; r <= sheet.getLastRowNum(); r++) {
//                Row row = sheet.getRow(r);
//                if (row == null) {
//                    continue;
//                }
//                List<List<String>> cellList = new ArrayList<>(row.getLastCellNum());
//                for (int c = 0; c < row.getLastCellNum(); c++) {
//                    Cell cell = row.getCell(c);
//                    if (cell == null) {
//                        continue;
//                    }
//                    String colString = CellReference.convertNumToColString(cell.getColumnIndex());
//                    System.out.println(colString);
//
//
//                    String stringCellValue = POIExcelUtil.getStringCellValExcludeBlank(cell);
//                    if (r == 0) {
//                        // 第一行有姓名列
//                        if ("姓名".equals(stringCellValue)) {
//                            nameCellIndex = c;
//                            break;
//                        } else {
//                            // 第一行没有指明姓名列
//                        }
//                    }
//                }
//                if (nameCellIndex != null) {
//                    Cell cell = row.getCell(nameCellIndex);
//                    String stringCellValue = POIExcelUtil.getStringCellValExcludeBlank(cell);
//                    if (!"姓名".equals(stringCellValue) && StrUtil.isNotBlank(stringCellValue)) {
//                        GmhUser user = new GmhUser();
//                        user.setName(stringCellValue);
//                        nameList.add(user);
//                    }
//                } else {
//                    System.out.println("未找到姓名列");
//                }
//            }
//            if (!nameList.isEmpty()) {
//                ReadNameVo readNameVo = new ReadNameVo();
//                readNameVo.setSheetName(sheet.getSheetName());
//                readNameVo.setNameList(nameList);
//                readNameList.add(readNameVo);
//            }
//        }
//        return R.ok().setData(readNameList);
    }

}
