package com.gmh.utils;

import cn.hutool.core.util.StrUtil;
import cn.hutool.poi.excel.ExcelFileUtil;
import com.gmh.entity.RowCellNum;
import com.sun.org.apache.bcel.internal.generic.IF_ACMPEQ;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellReference;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.io.*;
import java.nio.file.Files;
import java.util.*;

/**
 * POI Excel工具类
 * created on 2020/5/28 10:08
 *
 * @author puhaiyang
 */
public class POIExcelUtil {

    public static void main(String[] args) throws IOException {
        File file = new File("C:\\Users\\xxche\\Desktop\\20221106_test_excel\\附件3.继续教育证明.xls");
        Workbook workbook = batchCloneSheet(file, 0, 50);
        try {
            FileOutputStream out = new FileOutputStream("C:\\Users\\xxche\\Desktop\\20221106_test_excel\\附件3.继续教育证明222.xls");
            out.flush();
            workbook.write(out);
            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static Workbook cloneSheet(Workbook workbook, String srcSheetName, String destSheetName) {
        int index = workbook.getSheetIndex(srcSheetName);
        return cloneSheet(workbook, index, destSheetName);
    }

    public static Workbook cloneSheet(Workbook workbook, Integer index, String destSheetName) {
        //克隆一个新的sheet
        Sheet newSheet = workbook.cloneSheet(index);
        int sheetIndex = workbook.getSheetIndex(newSheet);
        workbook.setSheetName(sheetIndex, destSheetName);
        return workbook;
    }

    public static Workbook cloneSheet(File excelFile, Integer index, String destSheetName) throws IOException {
        Workbook workbook = readExcelFromFile(excelFile);
        return cloneSheet(workbook, index, destSheetName);
    }

    /**
     * 复制多个sheet
     *
     * @param workbook workbook
     * @param index    起始sheet
     * @param total    复制多个sheet
     */
    public static Workbook batchCloneSheet(Workbook workbook, Integer index, Integer total) throws IOException {
        for (int i = 0; i < total; i++) {
            //克隆一个新的sheet
            Sheet newSheet = workbook.cloneSheet(index);
            int sheetIndex = workbook.getSheetIndex(newSheet);
            workbook.setSheetName(sheetIndex, String.valueOf(i + 1));
        }
        workbook.removeSheetAt(index);
        return workbook;
    }

    public static Workbook batchCloneSheet(File excelFile, Integer index, Integer total) throws IOException {
        Workbook workbook = readExcelFromFile(excelFile);
        return batchCloneSheet(workbook, index, total);
    }

    //读取excel
    public static Workbook readExcelFromFile(File file) throws IOException {
        if (file == null) {
            throw new RuntimeException("文件为空");
        }
        return readExcelFromInputStream(Files.newInputStream(file.toPath()), file.getName());
    }

    public static Workbook readExcelFromInputStream(InputStream inputStream) throws IOException {
        return readExcelFromInputStream(inputStream, null);
    }

    public static Workbook readExcelFromInputStream(InputStream inputStream, String fileName) throws IOException {
        if (StrUtil.isNotBlank(fileName)) {
            String[] split = fileName.split("\\.");
            if (split.length == 0 || split.length == 1) {
                throw new RuntimeException("文件格式错误");
            }
            String suffix = split[split.length - 1];
            if ("xls".equals(suffix)) {
                return new HSSFWorkbook(inputStream);
            } else if ("xlsx".equals(suffix)) {
                return new XSSFWorkbook(inputStream);
            } else {
                throw new RuntimeException("文件格式错误");
            }
        }
        if (ExcelFileUtil.isXls(inputStream)) {
            return new HSSFWorkbook(inputStream);
        } else if (ExcelFileUtil.isXlsx(inputStream)) {
            return new XSSFWorkbook(inputStream);
        } else {
            throw new RuntimeException("文件格式错误");
        }
    }

    public static String getStringCellValExcludeBlank(Cell cell) {
        return getStringCellVal(cell).replaceAll("\\s*", "");
    }

    public static String getStringCellVal(Cell cell) {
        DataFormatter dataFormatter = new DataFormatter();
        return dataFormatter.formatCellValue(cell);
    }

    /**
     * Excel 转换为List<Map<String, String>>对象
     *
     * @param workbook    workbook
     * @param sheetIndex  当前sheet页索引
     * @return List<Map<String, String>>
     */
    public static List<Map<String, String>> toListMap(Workbook workbook, int sheetIndex) {
        return toListMap(workbook, sheetIndex, -1);
    }

    /**
     * Excel 转换为List<Map<String, String>>对象
     *
     * @param workbook    workbook
     * @param sheetIndex  当前sheet页索引
     * @param titleRowNum 标题行索引，索引值<0则代表没有标题行
     * @return List<Map<String, String>>
     */
    public static List<Map<String, String>> toListMap(Workbook workbook, int sheetIndex, int titleRowNum) {
        return toListMap(workbook.getSheetAt(sheetIndex), titleRowNum);
    }

    /**
     * Excel 转换为List<Map<String, String>>对象
     *
     * @param sheet    workbook
     * @return List<Map<String, String>>
     */
    public static List<Map<String, String>> toListMap(Sheet sheet) {
        return toListMap(sheet, -1);
    }

    /**
     * Excel 转换为List<Map<String, String>>对象
     *
     * @param sheet  当前sheet页对象
     * @param titleRowNum 标题行索引，索引值<0则代表没有标题行
     * @return List<Map<String, String>>
     */
    public static List<Map<String, String>> toListMap(Sheet sheet, int titleRowNum) {
        List<Map<String, String>> list = new ArrayList<>();
        List<String> mapKeys = new ArrayList<>();
        if (titleRowNum >= 0) {
            Row titleRow = sheet.getRow(titleRowNum);
            if (titleRow == null) {
                return list;
            }
            for (int i = 0; i < titleRow.getLastCellNum(); i++) {
                Cell cell = titleRow.getCell(i);
                if (cell == null) {
                    break;
                }
                String key = getStringCellValExcludeBlank(cell);
                mapKeys.add(key);
            }
        } else {
            // 根据第一行列的数量去找下面所有的列
            Row titleRow = sheet.getRow(0);
            if (titleRow == null) {
                return list;
            }
            for (int i = 0; i < titleRow.getLastCellNum(); i++) {
                Cell cell = titleRow.getCell(i);
                if (cell == null) {
                    break;
                }
                String colString = CellReference.convertNumToColString(cell.getColumnIndex());
                mapKeys.add(colString);
            }
        }

        for (int i = 0; i <= sheet.getLastRowNum(); i++) {
            // 指定了标题列的情况下获取数据时跳过标题列
            if (titleRowNum >= 0 && i == titleRowNum) {
                continue;
            }
            Row row = sheet.getRow(i);
            if (row == null) {
                continue;
            }
            Map<String, String> map = new LinkedHashMap<>();
            for (int c = 0; c < mapKeys.size(); c++) {
                map.put(mapKeys.get(c), getStringCellValExcludeBlank(row.getCell(c)));
            }
            list.add(map);
        }
        return list;
    }

    /**
     * 删除模板表格(除了 noDelSheet之外所有的)
     */
    public static void removeModelSheet(Workbook wb, Sheet noDelSheet) {
        int sheetIndex = wb.getSheetIndex(noDelSheet);
        removeModelSheet(wb,sheetIndex);
    }

    /**
     * 删除模板表格(除了 name之外所有的)
     */
    public static void removeModelSheet(Workbook wb, String name) {
        int sheetIndex = wb.getSheetIndex(wb.getSheet(name));
        removeModelSheet(wb,sheetIndex);
    }

    /**
     * 删除模板表格(除了 id之外所有的)
     */
    public static void removeModelSheet(Workbook wb, int id) {
        int numberOfSheets = wb.getNumberOfSheets();
        for (int i = numberOfSheets - 1; i > -1; i--) {
            if (i != id) {
                wb.removeSheetAt(i);
            }
        }
        //设置默认显示第一页
        wb.setActiveSheet(0);
    }

    /**
     * 将单元格逻辑坐标转换为物理坐标
     * @param cellRowStr 字符串形式的单元格坐标 如：A3
     * @return
     */
    public static RowCellNum analysisCellRowNum(String cellRowStr) {
        return analysisCellRowNum(new RowCellNum().setCellRowStr(cellRowStr));
    }

    /**
     * 将单元格逻辑坐标转换为物理坐标
     * @param rowCellNum
     * @return
     */
    public static RowCellNum analysisCellRowNum(RowCellNum rowCellNum) {
        String cellStrNum = rowCellNum.getCellRowStr().replaceAll("[^a-z^A-Z]", "");
        String rowNum = rowCellNum.getCellRowStr().replaceAll("[^0-9]", "");
        int cellIndex = CellReference.convertColStringToIndex(cellStrNum);
        return rowCellNum.setCellNum(cellIndex).setRowNum(Integer.parseInt(rowNum) - 1);
    }
}